@@ -5,11 +5,11 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 //
 
-#ifndef INCLUDED_UHD_TYPES_RANGES_HPP
-#define INCLUDED_UHD_TYPES_RANGES_HPP
+#pragma once
 
 #include <uhd/config.hpp>
 #include <string>
+#include <type_traits>
 #include <vector>
 
 namespace uhd {
@@ -77,6 +77,14 @@ struct UHD_API meta_range_t : std::vector<range_t>
     meta_range_t(InputIterator first, InputIterator last)
         : std::vector<range_t>(first, last)
     { /* NOP */
+        // This is to avoid people accidentally doing silly things like:
+        // meta_range_t(0, 0)
+        // which probably was supposed to call meta_range_t(double, double, double)
+        // but actually calls this constructor.
+        static_assert(
+            !std::is_integral<typename std::decay<InputIterator>::type>::value,
+            "You can't pass integers to meta_range_t's constructor!"
+        );
     }
 
     /*!
@@ -113,5 +121,3 @@ typedef meta_range_t gain_range_t;
 typedef meta_range_t freq_range_t;
 
 } // namespace uhd
-
-#endif /* INCLUDED_UHD_TYPES_RANGES_HPP */
