@@ -1,22 +1,11 @@
 //
 // Copyright 2015 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
-#ifndef INCLUDED_UHD_USRP_DBOARD_EEPROM_H
-#define INCLUDED_UHD_USRP_DBOARD_EEPROM_H
+#pragma once
 
 #include <uhd/config.h>
 #include <uhd/error.h>
@@ -110,5 +99,3 @@ UHD_API uhd_error uhd_dboard_eeprom_last_error(
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* INCLUDED_UHD_USRP_DBOARD_EEPROM_H */
