@@ -4,8 +4,7 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 //
 
-#ifndef INCLUDED_UHD_VERSION_H
-#define INCLUDED_UHD_VERSION_H
+#pragma once
 
 #include <uhd/config.h>
 #include <uhd/error.h>
@@ -31,5 +30,3 @@ UHD_API uhd_error uhd_get_version_string(
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* INCLUDED_UHD_VERSION_H */
