@@ -29,20 +29,19 @@ std::string _get_which(const direction_t dir, const size_t chan)
 magnesium_ad9371_iface::magnesium_ad9371_iface(
     uhd::rpc_client::sptr rpcc, const size_t slot_idx)
     : _rpcc(rpcc)
-    , _slot_idx(slot_idx)
     , _rpc_prefix((slot_idx == 0) ? "db_0_" : "db_1_")
     , _log_prefix((slot_idx == 0) ? "AD9371-0" : "AD9371-1")
 {
     UHD_LOG_TRACE(_log_prefix,
         "Initialized controls with RPC prefix " << _rpc_prefix << " for slot "
-                                                << _slot_idx);
+                                                << slot_idx);
 }
 
 double magnesium_ad9371_iface::set_frequency(
     const double freq, const size_t chan, const direction_t dir)
 {
     // Note: This sets the frequency for both channels (1 and 2).
-    auto which       = _get_which(dir, chan);
+    auto which = _get_which(dir, chan);
     auto actual_freq =
         request<double>(MAGNESIUM_TUNE_TIMEOUT, "set_freq", which, freq, false);
     UHD_LOG_TRACE(_log_prefix, _rpc_prefix << "set_freq returned " << actual_freq);
@@ -60,15 +59,6 @@ double magnesium_ad9371_iface::set_gain(
     // return 0.0;
 }
 
-
-double magnesium_ad9371_iface::set_master_clock_rate(const double freq)
-{
-    const auto actual_freq = request<double>("set_master_clock_rate", freq);
-    UHD_LOG_TRACE(
-        _log_prefix, _rpc_prefix << "set_master_clock_rate returned successfully");
-    return actual_freq;
-}
-
 double magnesium_ad9371_iface::set_bandwidth(
     const double bandwidth, const size_t chan, const direction_t dir)
 {
