@@ -86,6 +86,7 @@ public:
     std::future<void> handle_finish_initialization;
     std::future<void> handle_setup_cal;
     std::future<double> handle_set_freq;
+    std::future<void> handle_set_master_clock_rate;
 
     /*! \brief make a new AD9371 ctrl object using the specified SPI iface
      *
@@ -283,6 +284,20 @@ void export_mykonos(py::module& top_module)
 
     py::class_<ad937x_ctrl, std::shared_ptr<ad937x_ctrl>>(m, "ad937x_ctrl")
         .def("set_master_clock_rate", &ad937x_ctrl::set_master_clock_rate)
+        .def("async__set_master_clock_rate",
+            +[](ad937x_ctrl& self, const double rate) {
+                self.handle_set_master_clock_rate = std::async(std::launch::async,
+                    [&self, rate]() { self.set_master_clock_rate(rate); });
+            })
+        .def("await__set_master_clock_rate",
+            +[](ad937x_ctrl& self) -> bool {
+                if (self.handle_set_master_clock_rate.wait_for(std::chrono::seconds(0))
+                    == std::future_status::ready) {
+                    self.handle_set_master_clock_rate.get();
+                    return true;
+                }
+                return false;
+            })
         .def("begin_initialization", &ad937x_ctrl::begin_initialization)
         .def("async__finish_initialization",
             +[](ad937x_ctrl& self) {
@@ -345,30 +360,27 @@ void export_mykonos(py::module& top_module)
         .def("set_clock_rate", &ad937x_ctrl::set_clock_rate)
         .def("enable_channel", &ad937x_ctrl::enable_channel)
         .def("set_freq", &ad937x_ctrl::set_freq)
-        .def("async__set_freq", +[](
-                ad937x_ctrl& self,
-                const std::string &which,
-                const double value,
-                const bool wait_for_lock
-        ){
+        .def("async__set_freq",
+            +[](ad937x_ctrl& self,
+                 const std::string& which,
+                 const double value,
+                 const bool wait_for_lock) {
                 self.handle_set_freq = std::async(std::launch::async,
                     &ad937x_ctrl::set_freq,
                     &self,
                     which,
                     value,
-                    wait_for_lock
-                );
-        })
-        .def("await__set_freq", +[](
-                ad937x_ctrl& self
-        )->bool{
+                    wait_for_lock);
+            })
+        .def("await__set_freq",
+            +[](ad937x_ctrl& self) -> bool {
                 if (self.handle_set_freq.wait_for(std::chrono::seconds(0))
-                        == std::future_status::ready){
+                    == std::future_status::ready) {
                     self.handle_set_freq.get();
                     return true;
                 }
                 return false;
-        })
+            })
         .def("get_freq", &ad937x_ctrl::get_freq)
         .def("get_lo_locked", &ad937x_ctrl::get_lo_locked)
         .def("set_fir", &ad937x_ctrl::set_fir)
@@ -382,11 +394,14 @@ void export_mykonos(py::module& top_module)
         .def_readonly_static("RX_GAIN_DELAY", &ad937x_ctrl::RX_GAIN_DELAY)
         .def_readonly_static("FLASH_CAL", &ad937x_ctrl::FLASH_CAL)
         .def_readonly_static("PATH_DELAY", &ad937x_ctrl::PATH_DELAY)
-        .def_readonly_static("TX_LO_LEAKAGE_INTERNAL", &ad937x_ctrl::TX_LO_LEAKAGE_INTERNAL)
-        .def_readonly_static("TX_LO_LEAKAGE_EXTERNAL", &ad937x_ctrl::TX_LO_LEAKAGE_EXTERNAL)
+        .def_readonly_static(
+            "TX_LO_LEAKAGE_INTERNAL", &ad937x_ctrl::TX_LO_LEAKAGE_INTERNAL)
+        .def_readonly_static(
+            "TX_LO_LEAKAGE_EXTERNAL", &ad937x_ctrl::TX_LO_LEAKAGE_EXTERNAL)
         .def_readonly_static("TX_QEC_INIT", &ad937x_ctrl::TX_QEC_INIT)
         .def_readonly_static("LOOPBACK_RX_LO_DELAY", &ad937x_ctrl::LOOPBACK_RX_LO_DELAY)
-        .def_readonly_static("LOOPBACK_RX_RX_QEC_INIT", &ad937x_ctrl::LOOPBACK_RX_RX_QEC_INIT)
+        .def_readonly_static(
+            "LOOPBACK_RX_RX_QEC_INIT", &ad937x_ctrl::LOOPBACK_RX_RX_QEC_INIT)
         .def_readonly_static("RX_LO_DELAY", &ad937x_ctrl::RX_LO_DELAY)
         .def_readonly_static("RX_QEC_INIT", &ad937x_ctrl::RX_QEC_INIT)
         .def_readonly_static("DPD_INIT", &ad937x_ctrl::DPD_INIT)
@@ -409,7 +424,8 @@ void export_mykonos(py::module& top_module)
         .def_readonly_static("TRACK_ORX1_QEC_SNLO", &ad937x_ctrl::TRACK_ORX1_QEC_SNLO)
         .def_readonly_static("TRACK_ORX2_QEC_SNLO", &ad937x_ctrl::TRACK_ORX2_QEC_SNLO)
         .def_readonly_static("TRACK_SRX_QEC", &ad937x_ctrl::TRACK_SRX_QEC)
-        .def_readonly_static("DEFAULT_INIT_CALS_MASKS", &ad937x_ctrl::DEFAULT_INIT_CALS_MASKS)
+        .def_readonly_static(
+            "DEFAULT_INIT_CALS_MASKS", &ad937x_ctrl::DEFAULT_INIT_CALS_MASKS)
         .def_readonly_static(
             "DEFAULT_TRACKING_CALS_MASKS", &ad937x_ctrl::DEFAULT_TRACKING_CALS_MASKS)
         .def_readonly_static(
