@@ -18,7 +18,7 @@ from usrp_mpm.dboard_manager.lmk_mg import LMK04828Mg
 from usrp_mpm.dboard_manager.mg_periphs import DboardClockControl
 from usrp_mpm.cores import ClockSynchronizer
 from usrp_mpm.cores import nijesdcore
-from usrp_mpm.mpmutils import async_exec
+from usrp_mpm.mpmutils import async_exec, str2bool
 
 INIT_CALIBRATION_TABLE = {"TX_BB_FILTER"              :   0x0001,
                           "ADC_TUNER"                 :   0x0002,
@@ -76,6 +76,11 @@ class MagnesiumInitManager(object):
     # Variable PPS delay before the RP/SP pulsers begin. Fixed value for the
     # N3xx devices.
     N3XX_INT_PPS_DELAY = 4
+    # JESD core default configuration.
+    JESD_DEFAULT_ARGS = {"bypass_descrambler": False,
+                         "lmfc_divider"      : 20,
+                         "rx_sysref_delay"   : 8,
+                         "tx_sysref_delay"   : 11}
 
     def __init__(self, mg_class, spi_ifaces):
         self.mg_class = mg_class
@@ -525,7 +530,7 @@ class MagnesiumInitManager(object):
             read_only=False
         ) as dboard_ctrl_regs:
             self.log.trace("Creating jesdcore object...")
-            jesdcore = nijesdcore.NIMgJESDCore(dboard_ctrl_regs, slot_idx)
+            jesdcore = nijesdcore.NIJESDCore(dboard_ctrl_regs, slot_idx, **self.JESD_DEFAULT_ARGS)
             # Now get cracking with the actual init sequence:
             self.log.trace("Creating dboard clock control object...")
             db_clk_control = DboardClockControl(dboard_ctrl_regs, self.log)
@@ -551,8 +556,8 @@ class MagnesiumInitManager(object):
             self.log.debug(
                 "Sample Clocks and Phase DAC Configured Successfully!")
             # Clocks and PPS are now fully active!
-            if args.get('skip_rfic', None) == None:
-                self.mykonos.set_master_clock_rate(master_clock_rate)
+            if args.get('skip_rfic', None) is None:
+                async_exec(self.mykonos, "set_master_clock_rate", master_clock_rate)
                 self.init_jesd(jesdcore, master_clock_rate, args)
             jesdcore = None # Help with garbage collection
             # That's all that requires access to the dboard regs!
@@ -597,9 +602,9 @@ class MagnesiumInitManager(object):
             # TODO: Maybe we can switch to digital loopback without running the
             # initialization. For now, force init when rfic_digital_loopback is
             # set because we're being conservative.
-            if bool(args.get('rfic_digital_loopback')):
-                self.log.debug("Using rfic_digital_loopback causes a full "
-                               "re-init sequence.")
+            if 'rfic_digital_loopback' in args:
+                self.log.debug("Using rfic_digital_loopback flag causes a "
+                               "full re-init sequence.")
                 fast_reinit = False
         # If we can't do fast re-init, start from scratch:
         if not fast_reinit:
@@ -616,7 +621,7 @@ class MagnesiumInitManager(object):
                 self.log.debug(
                     "{}={}".format(arg_key, args.get(arg_key, arg_default)))
             return True
-        if bool(args.get('rfic_digital_loopback')):
+        if str2bool(args.get('rfic_digital_loopback')):
             self.log.warning(
                 "RF Functionality Disabled: JESD204b digital loopback "
                 "enabled inside Mykonos!")
