@@ -1,5 +1,6 @@
 #
 # Copyright 2018 Ettus Research, a National Instruments Company
+# Copyright 2019 Ettus Research, a National Instruments Brand
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -7,12 +8,11 @@
 N3xx peripherals
 """
 
-import datetime
 from usrp_mpm import lib
 from usrp_mpm.sys_utils.sysfs_gpio import SysFSGPIO, GPIOBank
-from usrp_mpm.sys_utils.uio import UIO
 from usrp_mpm.sys_utils import i2c_dev
 from usrp_mpm.chips.ds125df410 import DS125DF410
+from usrp_mpm.periph_manager.common import MboardRegsCommon
 
 # Map register values to SFP transport types
 N3XX_SFP_TYPES = {
@@ -102,7 +102,7 @@ class TCA6424(object):
             self.pins = self.pins_list[1]
 
         default_val = 0x860101 if rev == 2 else 0x860780
-        self._gpios = SysFSGPIO({'label': 'tca6424', 'device/of_node/name': 'gpio'}, 0xFFF7FF, 0x86F7FF, default_val)
+        self._gpios = SysFSGPIO({'device/name': 'tca6424', 'device/of_node/name': 'gpio'}, 0xFFF7FF, 0x86F7FF, default_val)
 
     def set(self, name, value=None):
         """
@@ -160,26 +160,20 @@ class BackpanelGPIO(GPIOBank):
             0x7, # ddr
         )
 
-class MboardRegsControl(object):
+class MboardRegsControl(MboardRegsCommon):
     """
     Control the FPGA Motherboard registers
     """
-    # Motherboard registers
-    M_COMPAT_NUM    = 0x0000
-    MB_DATESTAMP    = 0x0004
-    MB_GIT_HASH     = 0x0008
-    MB_SCRATCH      = 0x000C
-    MB_NUM_CE       = 0x0010
-    MB_NUM_IO_CE    = 0x0014
-    MB_CLOCK_CTRL   = 0x0018
-    MB_XADC_RB      = 0x001C
-    MB_BUS_CLK_RATE = 0x0020
-    MB_BUS_COUNTER  = 0x0024
-    MB_SFP0_INFO    = 0x0028
-    MB_SFP1_INFO    = 0x002C
-    MB_GPIO_MASTER  = 0x0030
-    MB_GPIO_RADIO_SRC  = 0x0034
-    MB_XBAR_BASEPORT   = 0x0038
+    # pylint: disable=bad-whitespace
+    # Motherboard registers (on top of the ones in MboardRegsCommon)
+    MB_CLOCK_CTRL     = 0x0018
+    MB_XADC_RB        = 0x001C
+    MB_BUS_CLK_RATE   = 0x0020
+    MB_BUS_COUNTER    = 0x0024
+    MB_SFP0_INFO      = 0x0028
+    MB_SFP1_INFO      = 0x002C
+    MB_GPIO_MASTER    = 0x0030
+    MB_GPIO_RADIO_SRC = 0x0034
 
     # Bitfield locations for the MB_CLOCK_CTRL register.
     MB_CLOCK_CTRL_PPS_SEL_INT_10 = 0 # pps_sel is one-hot encoded!
@@ -192,33 +186,17 @@ class MboardRegsControl(object):
     MB_CLOCK_CTRL_MEAS_CLK_RESET = 12 # set to 1 to reset mmcm, default is 0
     MB_CLOCK_CTRL_MEAS_CLK_LOCKED = 13 # locked indication for meas_clk mmcm
     MB_CLOCK_CTRL_DISABLE_REF_CLK = 16 # to disable the ref_clk, write a '1'
+    # pylint: enable=bad-whitespace
 
     def __init__(self, label, log):
-        self.log = log
-        self.regs = UIO(
-            label=label,
-            read_only=False
-        )
-        self.poke32 = self.regs.poke32
-        self.peek32 = self.regs.peek32
-
-    def get_compat_number(self):
-        """get FPGA compat number
-
-        This function reads back FPGA compat number.
-        The return is a tuple of
-        2 numbers: (major compat number, minor compat number )
-        """
-        with self.regs:
-            compat_number = self.peek32(self.M_COMPAT_NUM)
-        minor = compat_number & 0xff
-        major = (compat_number>>16) & 0xff
-        return (major, minor)
+        MboardRegsCommon.__init__(self, label, log)
 
     def set_fp_gpio_master(self, value):
         """set driver for front panel GPIO
         Arguments:
             value {unsigned} -- value is a single bit bit mask of 12 pins GPIO
+           0: means the pin is driven by PL
+           1: means the pin is driven by PS
         """
         with self.regs:
             return self.poke32(self.MB_GPIO_MASTER, value)
@@ -235,7 +213,7 @@ class MboardRegsControl(object):
     def set_fp_gpio_radio_src(self, value):
         """set driver for front panel GPIO
         Arguments:
-            value {unsigned} -- value is 2-bit bit mask of 12 pins GPIO
+            value {unsigned} -- value is 2x12 bits, two bits per GPIO pin
            00: means the pin is driven by radio 0
            01: means the pin is driven by radio 1
            10: means the pin is driven by radio 2
@@ -255,43 +233,6 @@ class MboardRegsControl(object):
         with self.regs:
             return self.peek32(self.MB_GPIO_RADIO_SRC) & 0xffffff
 
-    def get_build_timestamp(self):
-        """
-        Returns the build date/time for the FPGA image.
-        The return is datetime string with the  ISO 8601 format
-        (YYYY-MM-DD HH:MM:SS.mmmmmm)
-        """
-        with self.regs:
-            datestamp_rb = self.peek32(self.MB_DATESTAMP)
-        if datestamp_rb > 0:
-            dt_str = datetime.datetime(
-                year=((datestamp_rb>>17)&0x3F)+2000,
-                month=(datestamp_rb>>23)&0x0F,
-                day=(datestamp_rb>>27)&0x1F,
-                hour=(datestamp_rb>>12)&0x1F,
-                minute=(datestamp_rb>>6)&0x3F,
-                second=((datestamp_rb>>0)&0x3F))
-            self.log.trace("FPGA build timestamp: {}".format(str(dt_str)))
-            return str(dt_str)
-        else:
-            # Compatibility with FPGAs without datestamp capability
-            return ''
-
-    def get_git_hash(self):
-        """
-        Returns the GIT hash for the FPGA build.
-        The return is a tuple of
-        2 numbers: (short git hash, bool: is the tree dirty?)
-        """
-        with self.regs:
-            git_hash_rb = self.peek32(self.MB_GIT_HASH)
-        git_hash = git_hash_rb & 0x0FFFFFFF
-        tree_dirty = ((git_hash_rb & 0xF0000000) > 0)
-        dirtiness_qualifier = 'dirty' if tree_dirty else 'clean'
-        self.log.trace("FPGA build GIT Hash: {:07x} ({})".format(
-            git_hash, dirtiness_qualifier))
-        return (git_hash, dirtiness_qualifier)
-
     def set_time_source(self, time_source, ref_clk_freq):
         """
         Set time source
@@ -416,14 +357,15 @@ class MboardRegsControl(object):
                              .format(sfp0_type, sfp1_type))
         return ""
 
-    def get_xbar_baseport(self):
-        "Get the RFNoC crossbar base port"
-        with self.regs:
-            return self.peek32(self.MB_XBAR_BASEPORT)
 
 class RetimerQSFP(DS125DF410):
+    """
+    Thin wrapper around an I2C device that controls the QSFP retimer
+    """
     # (deemphasis, swing)
-    DRIVER_PRESETS = { '1m': (0x00, 0x07), '3m': (0x41, 0x06), 'Optical': (0x41, 0x04) }
+    DRIVER_PRESETS = {
+        '1m': (0x00, 0x07), '3m': (0x41, 0x06), 'Optical': (0x41, 0x04)
+    }
 
     def __init__(self, i2c_bus):
         regs_iface = lib.i2c.make_i2cdev_regs_iface(
